package org.dive4elements.river.exports.injector;

import org.apache.log4j.Logger;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.artifacts.ContextInjector;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.access.RiverAccess;
import org.dive4elements.river.model.Gauge;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import static org.dive4elements.river.exports.injector.InjectorConstants.CURRENT_KM;
import static org.dive4elements.river.exports.injector.InjectorConstants.PNP;
import static org.dive4elements.river.exports.injector.InjectorConstants.GAUGE_EPSILON;

public class PNPInjector
implements ContextInjector
{

    private Logger log = Logger.getLogger(PNPInjector.class);
    @Override
    public void setup(Element cfg) {
        // TODO Auto-generated method stub

    }

    @Override
    public void injectContext(
        CallContext ctx,
        Artifact artifact,
        Document request
    ) {
        Object currentKm = ctx.getContextValue(CURRENT_KM);
        if (currentKm == null) {
            log.debug("no current km. not injecting pnp");
            return;
        }

        RiverAccess access = new RiverAccess((D4EArtifact)artifact);
        Double km = Double.valueOf(currentKm.toString());
        if (Double.isNaN(km) || Double.isInfinite(km)) {
            log.debug("current km is NAN or infinte. not injecting pnp.");
            return;
        }
        Gauge gauge =
            access.getRiver().determineGaugeByStation(
                km - GAUGE_EPSILON,
                km + GAUGE_EPSILON);

        if (gauge == null) {
            log.debug("no gauge found at current km. not injecting pnp");
            return;
        }
        log.debug("injecting pnp: " + gauge.getDatum());
        ctx.putContextValue(PNP, gauge.getDatum());
    }

}
